import 'package:oobium_datastore/src/state.dart';

abstract class Model<T extends StateValue, I> extends StateKey<T> implements StateValue<T> {

  @override
  final I id;
  Model(this.id);

  T copyWith({I id});

  Map<String, dynamic> toJson();
}

abstract class ModelRef<T extends Model<T, I>, I, R extends ModelRef<T, I, R>> extends StateKey<T> {

  @override
  final I id;
  final WatchableStateFeature _feature;
  ModelRef(this._feature, this.id);

  bool get isNull => !isNotNull;
  bool get isNotNull => _feature.contains(this);

  I operator <<(T model) {
    _feature.put(model.copyWith(id: id));
    return id;
  }

  R load(T model) {
    _feature.put(model.copyWith(id: id));
    return this as R;
  }

  void notify() => _feature.flushEvents();

  T? remove() => _feature.remove(this);

  T toModel() => _feature.get(this);

  Map<String, dynamic> toJson() => toModel().toJson();

  Stream<R> stream({bool immediate = true}) async* {
    if (immediate && _feature.contains(this)) {
      yield this as R;
    }
    yield* watch()
        .takeWhile((element) => _feature.contains(this))
        .map((_) => this as R);
  }

  Stream<StateEvents> watch() {
    return _feature.watch().where((e) => e.contains(this));
  }
}

abstract class Singleton<T extends StateValue> extends StateKey<T> implements StateValue<T> {

  @override
  final String id = '$T';

  Map<String, dynamic> toJson();
}

abstract class SingletonRef<T extends StateValue, R extends SingletonRef<T, R>> extends StateKey<T> {

  @override
  final String id = '$T';
  final WatchableStateFeature _feature;
  SingletonRef(this._feature) : super();

  bool get isInitialized => _feature.contains(this);
  bool get isNotInitialized => !isInitialized;

  String operator <<(T state) {
    _feature.put(state);
    return state.id;
  }

  R load(T state) {
    _feature.put(state);
    return this as R;
  }

  void notify() => _feature.flushEvents();

  T toModel() => _feature.get(this);

  Stream<R> stream({bool immediate = true}) async* {
    if (immediate && _feature.contains(this)) {
      yield this as R;
    }
    yield* watch()
        .takeWhile((element) => _feature.contains(this))
        .map((_) => this as R);
  }

  Stream<StateEvents> watch() {
    return _feature.watch().where((e) => e.contains(this));
  }
}
