part of 'models_test.schema.g.dart';

class _RestFeature extends BufferedStateFeature {
  final SettingsRestAdapter settingsRestAdapter;
  final ConfigRestAdapter configRestAdapter;
  final AccountRestAdapter accountRestAdapter;
  final ImageRestAdapter imageRestAdapter;
  final UserRestAdapter userRestAdapter;
  final PostRestAdapter postRestAdapter;
  final CommentRestAdapter commentRestAdapter;
  _RestFeature({
    required this.settingsRestAdapter,
    required this.configRestAdapter,
    required this.accountRestAdapter,
    required this.imageRestAdapter,
    required this.userRestAdapter,
    required this.postRestAdapter,
    required this.commentRestAdapter,
  }) {
    settingsRestAdapter._feature = this;
    configRestAdapter._feature = this;
    imageRestAdapter._feature = this;
    userRestAdapter._feature = this;
    postRestAdapter._feature = this;
    commentRestAdapter._feature = this;
  }

  @override
  Future<void> onClose() {
    _reset();
    return super.onClose();
  }

  @override
  Future<void> onDestroy() {
    _reset();
    return super.onDestroy();
  }

  @override
  void onEvents(StateEvents events) {
    for (final event in events.created(this)) {
      if (event is StateEvent<Settings>) {
        _log.debug(() => 'created-Settings');
        if (settingsRestAdapter.onCreated != null) {
          q + (ex) => settingsRestAdapter._created(ex, event.value);
        }
      } else if (event is StateEvent<Config>) {
        _log.debug(() => 'created-Config');
        if (configRestAdapter.onCreated != null) {
          q + (ex) => configRestAdapter._created(ex, event.value);
        }
      } else if (event is StateEvent<Account>) {
        _log.debug(() => 'created-Account');
        if (accountRestAdapter.onCreated != null) {
          q + (ex) => accountRestAdapter._created(ex, event.value);
        }
      } else if (event is StateEvent<Image>) {
        _log.debug(() => 'created-Image');
        if (imageRestAdapter.onCreated != null) {
          q + (ex) => imageRestAdapter._created(ex, event.value);
        }
      } else if (event is StateEvent<User>) {
        _log.debug(() => 'created-User');
        if (userRestAdapter.onCreated != null) {
          q + (ex) => userRestAdapter._created(ex, event.value);
        }
      } else if (event is StateEvent<Post>) {
        _log.debug(() => 'created-Post');
        if (postRestAdapter.onCreated != null) {
          q + (ex) => postRestAdapter._created(ex, event.value);
        }
      } else if (event is StateEvent<Comment>) {
        _log.debug(() => 'created-Comment');
        if (commentRestAdapter.onCreated != null) {
          q + (ex) => commentRestAdapter._created(ex, event.value);
        }
      } else {
        _log.warn(() => 'created-UnknownType: $event');
      }
    }
    for (final event in events.updated(this)) {
      if (event is StateEvent<Settings>) {
        _log.debug(() => 'updated-Settings');
        if (settingsRestAdapter.onUpdated != null) {
          q +
              (ex) => settingsRestAdapter._updated(
                  ex, event.oldValue, event.newValue);
        }
      } else if (event is StateEvent<Config>) {
        _log.debug(() => 'updated-Config');
        if (configRestAdapter.onUpdated != null) {
          q +
              (ex) => configRestAdapter._updated(
                  ex, event.oldValue, event.newValue);
        }
      } else if (event is StateEvent<Account>) {
        _log.debug(() => 'updated-Account');
        if (accountRestAdapter.onUpdated != null) {
          q +
              (ex) => accountRestAdapter._updated(
                  ex, event.oldValue, event.newValue);
        }
      } else if (event is StateEvent<Image>) {
        _log.debug(() => 'updated-Image');
        if (imageRestAdapter.onUpdated != null) {
          q +
              (ex) =>
                  imageRestAdapter._updated(ex, event.oldValue, event.newValue);
        }
      } else if (event is StateEvent<User>) {
        _log.debug(() => 'updated-User');
        if (userRestAdapter.onUpdated != null) {
          q +
              (ex) =>
                  userRestAdapter._updated(ex, event.oldValue, event.newValue);
        }
      } else if (event is StateEvent<Post>) {
        _log.debug(() => 'updated-Post');
        if (postRestAdapter.onUpdated != null) {
          q +
              (ex) =>
                  postRestAdapter._updated(ex, event.oldValue, event.newValue);
        }
      } else if (event is StateEvent<Comment>) {
        _log.debug(() => 'updated-Comment');
        if (commentRestAdapter.onUpdated != null) {
          q +
              (ex) => commentRestAdapter._updated(
                  ex, event.oldValue, event.newValue);
        }
      } else {
        _log.warn(() => 'updated-UnknownType: $event');
      }
    }
    for (final event in events.removed(this)) {
      if (event is StateEvent<Account>) {
        _log.debug(() => 'removed-Account');
        if (accountRestAdapter.onRemoved != null) {
          q + (ex) => accountRestAdapter._removed(ex, event.value);
        }
      } else if (event is StateEvent<Image>) {
        _log.debug(() => 'removed-Image');
        if (imageRestAdapter.onRemoved != null) {
          q + (ex) => imageRestAdapter._removed(ex, event.value);
        }
      } else if (event is StateEvent<User>) {
        _log.debug(() => 'removed-User');
        if (userRestAdapter.onRemoved != null) {
          q + (ex) => userRestAdapter._removed(ex, event.value);
        }
      } else if (event is StateEvent<Post>) {
        _log.debug(() => 'removed-Post');
        if (postRestAdapter.onRemoved != null) {
          q + (ex) => postRestAdapter._removed(ex, event.value);
        }
      } else if (event is StateEvent<Comment>) {
        _log.debug(() => 'removed-Comment');
        if (commentRestAdapter.onRemoved != null) {
          q + (ex) => commentRestAdapter._removed(ex, event.value);
        }
      } else {
        _log.warn(() => 'removed-UnknownType: $event');
      }
    }
  }

  void _reset() {
    settingsRestAdapter.reset();
    configRestAdapter.reset();
    imageRestAdapter.reset();
    userRestAdapter.reset();
    postRestAdapter.reset();
    commentRestAdapter.reset();
  }
}

class SettingsRestAdapter {
  late final _RestFeature _feature;
  FutureOr<Map<String, dynamic>?> Function(StateReader state)? onRequest;
  FutureOr<void> Function(StateWriter state, Map<String, dynamic> result)?
      onResults;
  FutureOr<void> Function(StateReader state, Settings settings)? onCreated;
  FutureOr<void> Function(
      StateReader state, Settings oldSettings, Settings settings)? onUpdated;
  Future<void>? _fetched;
  SettingsRestAdapter(
      {this.onRequest, this.onResults, this.onCreated, this.onUpdated});
  FutureOr<void> _created(Executor ex, Settings settings) =>
      onCreated?.call(_feature.asReader(), settings);
  FutureOr<void> _updated(
          Executor ex, Settings oldSettings, Settings settings) =>
      onUpdated?.call(_feature.asReader(), oldSettings, settings);
  void reset() => _fetched = null;
  Future<void> fetch(bool force) {
    if (force) {
      final current = _fetched;
      if (current != null) {
        _fetched = null;
        return current.then((_) => fetch(false));
      }
    }
    return (_fetched ??= _fetch());
  }

  Future<void> _fetch() async {
    _log.debug(() => 'fetching Settings');
    final request = onRequest;
    if (request == null) {
      _log.debug(() => 'fetched Settings: skipped, onRequest not set');
    } else {
      final data = await request(_feature.asReader());
      if (data == null) {
        _log.debug(() => 'fetched Settings: not found');
      } else {
        await onResults?.call(_feature.asWriter(), data);
        final settings = Settings.fromJson(data);
        _feature.put(settings);
        _log.debug(() => 'fetched $settings');
      }
    }
  }
}

class ConfigRestAdapter {
  late final _RestFeature _feature;
  FutureOr<Map<String, dynamic>?> Function(StateReader state)? onRequest;
  FutureOr<void> Function(StateWriter state, Map<String, dynamic> result)?
      onResults;
  FutureOr<void> Function(StateReader state, Config config)? onCreated;
  FutureOr<void> Function(StateReader state, Config oldConfig, Config config)?
      onUpdated;
  Future<void>? _fetched;
  ConfigRestAdapter(
      {this.onRequest, this.onResults, this.onCreated, this.onUpdated});
  FutureOr<void> _created(Executor ex, Config config) =>
      onCreated?.call(_feature.asReader(), config);
  FutureOr<void> _updated(Executor ex, Config oldConfig, Config config) =>
      onUpdated?.call(_feature.asReader(), oldConfig, config);
  void reset() => _fetched = null;
  Future<void> fetch(bool force) {
    if (force) {
      final current = _fetched;
      if (current != null) {
        _fetched = null;
        return current.then((_) => fetch(false));
      }
    }
    return (_fetched ??= _fetch());
  }

  Future<void> _fetch() async {
    _log.debug(() => 'fetching Config');
    final request = onRequest;
    if (request == null) {
      _log.debug(() => 'fetched Config: skipped, onRequest not set');
    } else {
      final data = await request(_feature.asReader());
      if (data == null) {
        _log.debug(() => 'fetched Config: not found');
      } else {
        await onResults?.call(_feature.asWriter(), data);
        final config = Config.fromJson(data);
        _feature.put(config);
        _log.debug(() => 'fetched $config');
      }
    }
  }
}

class AccountRestAdapter {
  late final _RestFeature _feature;
  FutureOr<void> Function(StateReader state, Account account)? onCreated;
  FutureOr<void> Function(StateReader state, Account account)? onRemoved;
  FutureOr<void> Function(
      StateReader state, Account oldAccount, Account account)? onUpdated;
  AccountRestAdapter({this.onCreated, this.onRemoved, this.onUpdated});
  FutureOr<void> _created(Executor ex, Account account) =>
      onCreated?.call(_feature.asReader(), account);
  FutureOr<void> _removed(Executor ex, Account account) =>
      onRemoved?.call(_feature.asReader(), account);
  FutureOr<void> _updated(Executor ex, Account oldAccount, Account account) =>
      onUpdated?.call(_feature.asReader(), oldAccount, account);
}

class ImageRestAdapter {
  late final _RestFeature _feature;
  FutureOr<dynamic> Function(StateReader state, ImageQuery query)? onRequest;
  FutureOr<void> Function(StateWriter state, ImageQuery query, dynamic results)?
      onResults;
  FutureOr<void> Function(StateReader state, Image image)? onCreated;
  FutureOr<void> Function(StateReader state, Image image)? onRemoved;
  FutureOr<void> Function(StateReader state, Image oldImage, Image image)?
      onUpdated;
  final _fetched = <int, Future<void>>{};
  ImageRestAdapter(
      {this.onRequest,
      this.onResults,
      this.onCreated,
      this.onRemoved,
      this.onUpdated});
  FutureOr<void> _created(Executor ex, Image image) =>
      onCreated?.call(_feature.asReader(), image);
  FutureOr<void> _removed(Executor ex, Image image) =>
      onRemoved?.call(_feature.asReader(), image);
  FutureOr<void> _updated(Executor ex, Image oldImage, Image image) =>
      onUpdated?.call(_feature.asReader(), oldImage, image);
  void reset() => _fetched.clear();
  Future<void> fetch(ImageQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(ImageQuery query) async {
    _log.debug(() => 'fetching Image($query)');
    final request = onRequest;
    if (request == null) {
      _log.debug(() => 'fetched Image: skipped, onRequest not set');
    } else {
      final data = await request(_feature.asReader(), query);
      if (data == null) {
        _log.debug(() => 'fetched Image: not found');
      } else if (data is Map<String, dynamic>) {
        await onResults?.call(_feature.asWriter(), query, data);
        final image = Image.fromJson(data, withQuery: query);
        _feature.put(image);
        _log.debug(() => 'fetched $image');
      } else if (data is Iterable) {
        await onResults?.call(_feature.asWriter(), query, data);
        final images = Image.fromJsonList(data, withQuery: query);
        _feature.set<Image>(images, where: query.test);
        _log.debug(() => 'fetched $images');
      } else {
        _log.debug(() => 'fetched Image: invalid result($data)');
      }
    }
  }
}

class UserRestAdapter {
  late final _RestFeature _feature;
  FutureOr<dynamic> Function(StateReader state, UserQuery query)? onRequest;
  FutureOr<void> Function(StateWriter state, UserQuery query, dynamic results)?
      onResults;
  FutureOr<void> Function(StateReader state, User user)? onCreated;
  FutureOr<void> Function(StateReader state, User user)? onRemoved;
  FutureOr<void> Function(StateReader state, User oldUser, User user)?
      onUpdated;
  final _fetched = <int, Future<void>>{};
  UserRestAdapter(
      {this.onRequest,
      this.onResults,
      this.onCreated,
      this.onRemoved,
      this.onUpdated});
  FutureOr<void> _created(Executor ex, User user) =>
      onCreated?.call(_feature.asReader(), user);
  FutureOr<void> _removed(Executor ex, User user) =>
      onRemoved?.call(_feature.asReader(), user);
  FutureOr<void> _updated(Executor ex, User oldUser, User user) =>
      onUpdated?.call(_feature.asReader(), oldUser, user);
  void reset() => _fetched.clear();
  Future<void> fetch(UserQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(UserQuery query) async {
    _log.debug(() => 'fetching User($query)');
    final request = onRequest;
    if (request == null) {
      _log.debug(() => 'fetched User: skipped, onRequest not set');
    } else {
      final data = await request(_feature.asReader(), query);
      if (data == null) {
        _log.debug(() => 'fetched User: not found');
      } else if (data is Map<String, dynamic>) {
        await onResults?.call(_feature.asWriter(), query, data);
        final user = User.fromJson(data, withQuery: query);
        _feature.put(user);
        _log.debug(() => 'fetched $user');
      } else if (data is Iterable) {
        await onResults?.call(_feature.asWriter(), query, data);
        final users = User.fromJsonList(data, withQuery: query);
        _feature.set<User>(users, where: query.test);
        _log.debug(() => 'fetched $users');
      } else {
        _log.debug(() => 'fetched User: invalid result($data)');
      }
    }
  }
}

class PostRestAdapter {
  late final _RestFeature _feature;
  FutureOr<dynamic> Function(StateReader state, PostQuery query)? onRequest;
  FutureOr<void> Function(StateWriter state, PostQuery query, dynamic results)?
      onResults;
  FutureOr<void> Function(StateReader state, Post post)? onCreated;
  FutureOr<void> Function(StateReader state, Post post)? onRemoved;
  FutureOr<void> Function(StateReader state, Post oldPost, Post post)?
      onUpdated;
  final _fetched = <int, Future<void>>{};
  PostRestAdapter(
      {this.onRequest,
      this.onResults,
      this.onCreated,
      this.onRemoved,
      this.onUpdated});
  FutureOr<void> _created(Executor ex, Post post) =>
      onCreated?.call(_feature.asReader(), post);
  FutureOr<void> _removed(Executor ex, Post post) =>
      onRemoved?.call(_feature.asReader(), post);
  FutureOr<void> _updated(Executor ex, Post oldPost, Post post) =>
      onUpdated?.call(_feature.asReader(), oldPost, post);
  void reset() => _fetched.clear();
  Future<void> fetch(PostQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(PostQuery query) async {
    _log.debug(() => 'fetching Post($query)');
    final request = onRequest;
    if (request == null) {
      _log.debug(() => 'fetched Post: skipped, onRequest not set');
    } else {
      final data = await request(_feature.asReader(), query);
      if (data == null) {
        _log.debug(() => 'fetched Post: not found');
      } else if (data is Map<String, dynamic>) {
        await onResults?.call(_feature.asWriter(), query, data);
        final post = Post.fromJson(data, withQuery: query);
        _feature.put(post);
        _log.debug(() => 'fetched $post');
      } else if (data is Iterable) {
        await onResults?.call(_feature.asWriter(), query, data);
        final posts = Post.fromJsonList(data, withQuery: query);
        _feature.set<Post>(posts, where: query.test);
        _log.debug(() => 'fetched $posts');
      } else {
        _log.debug(() => 'fetched Post: invalid result($data)');
      }
    }
  }
}

class CommentRestAdapter {
  late final _RestFeature _feature;
  FutureOr<dynamic> Function(StateReader state, CommentQuery query)? onRequest;
  FutureOr<void> Function(
      StateWriter state, CommentQuery query, dynamic results)? onResults;
  FutureOr<void> Function(StateReader state, Comment comment)? onCreated;
  FutureOr<void> Function(StateReader state, Comment comment)? onRemoved;
  FutureOr<void> Function(
      StateReader state, Comment oldComment, Comment comment)? onUpdated;
  final _fetched = <int, Future<void>>{};
  CommentRestAdapter(
      {this.onRequest,
      this.onResults,
      this.onCreated,
      this.onRemoved,
      this.onUpdated});
  FutureOr<void> _created(Executor ex, Comment comment) =>
      onCreated?.call(_feature.asReader(), comment);
  FutureOr<void> _removed(Executor ex, Comment comment) =>
      onRemoved?.call(_feature.asReader(), comment);
  FutureOr<void> _updated(Executor ex, Comment oldComment, Comment comment) =>
      onUpdated?.call(_feature.asReader(), oldComment, comment);
  void reset() => _fetched.clear();
  Future<void> fetch(CommentQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(CommentQuery query) async {
    _log.debug(() => 'fetching Comment($query)');
    final request = onRequest;
    if (request == null) {
      _log.debug(() => 'fetched Comment: skipped, onRequest not set');
    } else {
      final data = await request(_feature.asReader(), query);
      if (data == null) {
        _log.debug(() => 'fetched Comment: not found');
      } else if (data is Map<String, dynamic>) {
        await onResults?.call(_feature.asWriter(), query, data);
        final comment = Comment.fromJson(data, withQuery: query);
        _feature.put(comment);
        _log.debug(() => 'fetched $comment');
      } else if (data is Iterable) {
        await onResults?.call(_feature.asWriter(), query, data);
        final comments = Comment.fromJsonList(data, withQuery: query);
        _feature.set<Comment>(comments, where: query.test);
        _log.debug(() => 'fetched $comments');
      } else {
        _log.debug(() => 'fetched Comment: invalid result($data)');
      }
    }
  }
}
